/* jcard by dSoft  Version 1.1 */
/* files.c */

#include "jcard.h"

/* latex() - generate a latex file */
/****************************************************************************/
void latex (int title_lines,
	    char title[][MAX_TITLE],
	    char source[][MAX_BACK],
	    char gen[][MAX_BACK],
	    char nr[][MAX_BACK])
{
  extern Song *side_a;
  extern Song *side_b;

  Song *a_curr = side_a;
  Song *b_curr = side_b;

  FILE *lf = fopen("jcard.tex", "w");
  if (!lf) die ("Can't open jcard.tex for writing.");

  fprintf (stderr, "Creating LaTeX file... ");

  fputs ("\\documentclass{article}\n", lf);
  fputs ("\\begin{document}\n", lf);
  fputs ("\\setlength{\\parindent}{0cm}\n\n", lf);
  fputs ("% This mbox is the wrapper for the entire label.\n\n", lf);

  /* Label back */
  fputs ("  % This framebox is the ``back'' of the label.\n", lf);
  fputs ("  \\framebox[4in]{\n", lf);
  fputs ("    \\begin{minipage}[c][1.5cm]{3.8in}\n", lf);
  fprintf (lf, "      %s\\hfill Source\\hfill %s\\\\\n", source[0], source[1]);
  fprintf (lf, "      %s\\hfill Generation\\hfill %s\\\\\n", gen[0], gen[1]);
  fprintf (lf, "      %s\\hfill NR\\hfill %s\n", nr[0], nr[1]);
  fputs ("    \\end{minipage}}\\\\\n\n", lf);

  /* Label spine */
  fputs ("  % This framebox is the ``spine'' of the label.\n", lf);
  fputs ("  \\framebox[4in]{\n", lf);
  fputs ("    \\begin{minipage}[c][0.5in]{3.8in}\n", lf);
  fputs ("    \\begin{center}\n", lf);
  if (title_lines == 1)
    fprintf (lf, "    {\\huge %s}\n", title[0]);
  else
    fprintf (lf, "    {\\Large %s\\\\*[2mm]%s}\n", title[0], title[1]);
  fputs ("    \\end{center}\n", lf);
  fputs ("    \\end{minipage}}\\\\\n\n", lf);

  /* Songs area */
  fputs ("  % This framebox is the song area of the label.\n", lf);
  fputs ("  \\framebox[4in]{\n", lf);
  fputs ("    \\begin{minipage}[t][6.5cm]{3.8in}\n", lf);
  fputs ("      {\\Large A}\\hfill {\\Large B}\\\\\n", lf);

  /* While there are songs left on both sides */
  while (a_curr->title[0] && b_curr->title[0])
    {
      fprintf (lf, "      %s\\hfill %s\\\\\n", a_curr->title, b_curr->title);
      a_curr = a_curr->next;
      b_curr = b_curr->next;
    } /* while both sides have songs left */

  while (a_curr->title[0]) /* There are songs left on side A */
    {
      fprintf (lf, "      %s\\\\\n", a_curr->title);
      a_curr = a_curr->next;
    } /* while */

  while (b_curr->title[0]) /* There are songs left on side B */
    {
      fprintf (lf, "      \\hspace*{\\fill}%s\\\\\n", b_curr->title);
      b_curr = b_curr->next;
    } /* while */
  fprintf (lf, "  \\end{minipage}}\n");
  fprintf (lf, "\\end{document}\n");

  /* Close the latex file */
  fclose(lf);
  fprintf (stderr, "done.\n");
} /* latex */

/* dvi() - generates a .dvi file from the .tex file created in latex() */
/****************************************************************************/
void dvi (void)
{
  int return_code;

  fprintf (stderr, "Creating dvi file... ");
  return_code =  system("latex jcard >& /dev/null");
  if (return_code) die ("Can't create .dvi file.");
  fprintf (stderr, "done.\n");
  return;
} /* dvi */

/* ps() - generates a postscript file from the .dvi file created in dvi() */
/****************************************************************************/
void ps (void)
{
  int return_code;

  fprintf (stderr, "Creating PostScript file... ");
  return_code = system("dvips -o jcard.ps jcard.dvi >& /dev/null");
  if (return_code) die ("Can't create .ps file.");
  fprintf (stderr, "done.\njcard.ps created.\n");
  return;
} /* ps */

/* openfile() - Reads a data file to create a jcard */
/****************************************************************************/
void openfile (char *filename, int *title_lines, char title[][MAX_TITLE],
	       char source[][MAX_BACK], char gen[][MAX_BACK],
	       char nr[][MAX_BACK])
{
  extern Song *side_a;
  extern Song *side_b;
  FILE *data;

  /* Open data file for reading */
  fprintf (stderr, "Opening data file %s... ", filename);
  data = fopen (filename, "rt");
  if (!data) die ("Can't open data file.");
  printf ("done.\n");

  /* Read in the title */
  fgetline (data, title[0], MAX_TITLE);
  fgetline (data, title[1], MAX_TITLE);
  *title_lines = 1 + (title[1][0] != 0);

  /* Read in back of jcard information */
  fgetline (data, source[0], MAX_BACK);
  fgetline (data, source[1], MAX_BACK);
  fgetline (data, gen[0], MAX_BACK);
  fgetline (data, gen[1], MAX_BACK);
  fgetline (data, nr[0], MAX_BACK);
  fgetline (data, nr[1], MAX_BACK);

  side_a = songs(data, 'A');
  side_b = songs(data, 'B');
  fclose (data);
  return;
} /* openfile */
  
/* savefile() - Creates a data file to save a jcard configuration */
/****************************************************************************/
void savefile (int lines, char title[][MAX_TITLE], char source[][MAX_BACK],
	       char gen[][MAX_BACK], char nr[][MAX_BACK])
{
  extern Song *side_a;
  extern Song *side_b;
  FILE *data;
  char filename[MAX_FILENAME];
  char in_file[MAX_FILENAME];
  int i;

  /* Create default file name */
  for (i = 0; title[0][i] && title[0][i] != ' '; i++)
    filename[i] = title[0][i];
  filename[i] = 0;
  strcat(filename, ".jcard");

  /* Ask for filename */
  printf ("File name for save file <Default: %s>: ", filename);
  fgetline (stdin, in_file, MAX_FILENAME);
  if (in_file[0]) strcpy (filename, in_file);

  /* Open data file for writing */
  fprintf (stderr, "Writing to data file %s... ", filename);
  data = fopen (filename, "w");
  if (!data) die ("Can't open data file.");
 
  /* Write the title */
  fprintf (data, "%s\n", title[0]);
  if (lines == 2)
    fprintf (data, "%s\n", title[1]);
  else
    fprintf (data, "\n");

  /* Write back information */
  fprintf (data, "%s\n%s\n", source[0], source[1]);
  fprintf (data, "%s\n%s\n", gen[0], gen[1]);
  fprintf (data, "%s\n%s\n", nr[0], nr[1]);

  /* Write song lists */
  while (side_a->title[0])
    {
      fprintf (data, "%s\n", side_a->title);
      side_a = side_a->next;
    } /* while */
  fprintf (data, "\n"); /* Null string to end side a */
  while (side_b->title[0])
    {
      fprintf (data, "%s\n", side_b->title);
      side_b = side_b->next;
    } /* while */
  fprintf (data, "\n"); /* Null string to end side b */
  fclose (data);
  fprintf (stderr, "done.\n");
} /* savefile */
  
